"use server";

import prisma from "@/lib/prisma";
import { s3 } from "@/lib/s3";
import { DeleteObjectCommand } from "@aws-sdk/client-s3";

export async function deleteImage(imageId: string) {
  const image = await prisma.image.findUnique({
    where: { id: imageId },
    include: {
      variants: true,
      palettes: true,
      colors: true,
      extractColors: true,
      metadata: true,
      stats: true,
      albumCover: true,
      galleryCover: true,
      tags: true,
      categories: true,
    },
  });

  if (!image) throw new Error("Image not found");

  // Delete S3 objects
  for (const variant of image.variants) {
    try {
      await s3.send(
        new DeleteObjectCommand({
          Bucket: "felliesartapp",
          Key: variant.s3Key,
        })
      );
    } catch (err) {
      console.warn("Failed to delete S3 object: " + variant.s3Key + ". " + err);
    }
  }

  // Step 1: Delete join entries
  await prisma.imagePalette.deleteMany({ where: { imageId } });
  await prisma.imageColor.deleteMany({ where: { imageId } });
  await prisma.imageExtractColor.deleteMany({ where: { imageId } });

  // ColorPalettes
  const connectedPalettes = image.palettes;
  for (const palette of connectedPalettes) {
    const count = await prisma.imagePalette.count({
      where: { paletteId: palette.id },
    });
    if (count === 0) {
      await prisma.colorPaletteItem.deleteMany({ where: { colorPaletteId: palette.id } });
      await prisma.colorPalette.deleteMany({ where: { id: palette.id } });
    }
  }

  // ExtractColors
  for (const extract of image.extractColors) {
    const count = await prisma.imageExtractColor.count({
      where: { extractId: extract.extractId },
    });
    if (count === 0) {
      await prisma.extractColor.delete({ where: { id: extract.extractId } });
    }
  }

  // Colors
  for (const color of image.colors) {
    const count = await prisma.imageColor.count({
      where: { colorId: color.colorId },
    });
    if (count === 0) {
      await prisma.color.delete({ where: { id: color.colorId } });
    }
  }

  // Delete variants
  await prisma.imageVariant.deleteMany({ where: { imageId } });

  // Delete metadata
  await prisma.imageMetadata.deleteMany({ where: { imageId } });
  await prisma.imageStats.deleteMany({ where: { imageId } });

  // Clean many-to-many tag/category joins
  await prisma.image.update({
    where: { id: imageId },
    data: {
      tags: { set: [] },
      categories: { set: [] },
    },
  });

  // Delete possible coverImage relation
  await prisma.album.updateMany({
    where: { coverImageId: imageId },
    data: { coverImageId: null },
  });
  await prisma.gallery.updateMany({
    where: { coverImageId: imageId },
    data: { coverImageId: null },
  });

  // Finally delete the image
  await prisma.image.delete({ where: { id: imageId } });

  return { success: true };
}