'use client';

import type { Color, Image, ImageColor } from '@/generated/prisma';
import { useSortable } from '@dnd-kit/sortable';
import { CSS } from '@dnd-kit/utilities';
import clsx from 'clsx';
import { GripVertical } from 'lucide-react';
import NextImage from 'next/image';
import Link from 'next/link';

type SupportedTypes = 'gallery' | 'album' | 'artist' | 'category' | 'tag';

const pluralMap: Record<SupportedTypes, string> = {
  gallery: 'galleries',
  album: 'albums',
  artist: 'artists',
  category: 'categories',
  tag: 'tags',
};

type SortableCardItemProps = {
  id: string;
  item: {
    id: string;
    name: string;
    type: 'gallery' | 'album' | 'artist' | 'category' | 'tag';
    coverImage?: (Image & { colors?: (ImageColor & { color: Color })[] }) | null;
    count?: number;
    textLabel?: string;
  };
};

export function SortableCardItem({ id, item }: SortableCardItemProps) {
  const { attributes, listeners, setNodeRef, transform, transition } = useSortable({ id });

  const style = {
    transform: CSS.Transform.toString(transform),
    transition,
  };

  const href = `/${pluralMap[item.type]}/edit/${item.id}`;

  const isVisualType = item.type === 'gallery' || item.type === 'album';

  let countLabel = '';
  if (item.count !== undefined) {
    if (item.type === 'gallery') {
      countLabel = `${item.count} album${item.count !== 1 ? 's' : ''}`;
    } else {
      countLabel = `${item.count} image${item.count !== 1 ? 's' : ''}`;
    }
  }

  return (
    <div
      ref={setNodeRef}
      style={style}
      {...attributes}
      className="relative cursor-grab active:cursor-grabbing"
    >
      <div
        {...listeners}
        className="absolute top-2 left-2 z-20 text-muted-foreground bg-white/70 rounded-full p-1"
        title="Drag to reorder"
      >
        <GripVertical className="w-4 h-4" />
      </div>

      <Link href={href}>
        <div className="group rounded-lg border overflow-hidden hover:shadow-md transition-shadow bg-background relative">
          {isVisualType ? (
            <div className="relative aspect-[4/3] w-full bg-muted items-center justify-center">
              {item.coverImage?.fileKey ? (
                <NextImage
                  src={`/api/image/thumbnails/${item.coverImage.fileKey}.webp`}
                  alt={item.coverImage.imageName}
                  fill
                  className={clsx("object-cover transition duration-300")}
                />
              ) : (
                <div className="flex items-center justify-center h-full text-muted-foreground text-sm">
                  No cover image
                </div>
              )}
            </div>
          ) : null}

          <div className={clsx("p-4", !isVisualType && "text-center")}>
            <h2 className={clsx("text-lg font-semibold truncate", isVisualType && "text-center")}>
              {item.name}
            </h2>
            {countLabel && (
              <p className="text-sm text-muted-foreground mt-1">{countLabel}</p>
            )}
            {item.textLabel && (
              <p className="text-sm text-muted-foreground mt-1">{item.textLabel}</p>
            )}
          </div>
        </div>
      </Link>
    </div>
  );
}
