'use client';

import {
  closestCenter,
  DndContext,
  DragEndEvent,
  KeyboardSensor,
  PointerSensor,
  useSensor,
  useSensors,
} from '@dnd-kit/core';
import {
  arrayMove,
  SortableContext,
  sortableKeyboardCoordinates,
  verticalListSortingStrategy,
} from '@dnd-kit/sortable';
import { useState } from 'react';

import type { SortableItem as ItemType } from '@/types/SortableItem';

interface Props {
  items: ItemType[];
  onReorder: (items: ItemType[]) => void;
  onSortDefault?: () => void;
  defaultSortLabel?: string;
  renderItem: (item: ItemType) => React.ReactNode;
}

export function SortableList({
  items,
  onReorder,
  onSortDefault,
  defaultSortLabel = 'Sort by Name',
  renderItem,
}: Props) {
  const [localItems, setLocalItems] = useState(items);

  const sensors = useSensors(
    useSensor(PointerSensor),
    useSensor(KeyboardSensor, {
      coordinateGetter: sortableKeyboardCoordinates,
    }),
  );

  const handleDragEnd = (event: DragEndEvent) => {
    const { active, over } = event;
    if (!over || active.id === over.id) return;

    const oldIndex = localItems.findIndex(item => item.id === active.id);
    const newIndex = localItems.findIndex(item => item.id === over.id);
    if (oldIndex === -1 || newIndex === -1) return;

    const reordered = arrayMove(localItems, oldIndex, newIndex).map((item, index) => ({
      ...item,
      sortIndex: index * 10,
    }));

    setLocalItems(reordered);
    onReorder(reordered);
  };

  return (
    <div className="space-y-4">
      {onSortDefault && (
        <button
          onClick={onSortDefault}
          className="px-4 py-2 text-sm font-medium bg-gray-200 rounded hover:bg-gray-300 text-primary-foreground"
        >
          {defaultSortLabel}
        </button>
      )}

      <DndContext sensors={sensors} collisionDetection={closestCenter} onDragEnd={handleDragEnd}>
        <SortableContext items={localItems.map(i => i.id)} strategy={verticalListSortingStrategy}>
          <div className="grid gap-6 sm:grid-cols-2 md:grid-cols-3 xl:grid-cols-4">
            {localItems.map(item => (
              <div key={item.id}>{renderItem(item)}</div>
            ))}
          </div>
        </SortableContext>
      </DndContext>
    </div>
  );
}
